classdef DatasetLoaderYork

    properties
        datasetPath
        imageNames
        count
        focal
        pp
    end
    
    methods
        
        function obj = DatasetLoaderYork()
            % Constructor, load common data
            datasetPath = textread('dataset_york_path.ini','%s\n');
            obj.datasetPath = datasetPath{1};
            if length(obj.datasetPath) < 1
                error('Please configure the path of the york dataset in the file dataset_york_path.ini');
            end
            files = dir(fullfile(obj.datasetPath, 'P*')); % All the images are in a foler with a name starting with 'P'
            obj.imageNames = {};
            j = 1;
            for i=1:length(files)
                if files(i).isdir
                    obj.imageNames{j} = files(i).name;
                    j = j + 1;
                end
            end
            obj.count = length(obj.imageNames);
            if obj.count < 1
                error(strcat('No image found in the folder ', obj.datasetPath));
            end  
            % Load intrinsics
            load ( fullfile(obj.datasetPath, 'cameraParameters.mat') );
            obj.focal = focal/pixelSize;
            obj.pp = pp;
        end
        
        function imageData=getImageDataAtIndex(obj, index)
            % Load the indexth image, lines and intrinsics
            % Return a struct imageData with the fields: 
            %   K: camera intrinsics parameters (depends on image orientation)
            %   Kinv: Kinv
            %   Kacc: accuracy on the terms of K
            %   img: image
            %   lines: gt lines segments in the format nx4 with lines(i,:) are the x,y coordinates of the two endpoints
            %   linesEq: gt lines equations 
            %   name: image name
            %   vp_association: to which vp a line is associated with
            %   vp: reference vanishing point in camera coordinates
            %   vp_img: reference vanishing point in image coordinates
            %   vp_ortho: reference vanshing points in camera coordinates after orthogonalization
            %   vp_ortho_img: reference vanshing points in image coordinates after orthogonalization
            if index > obj.count
                error('index is out of bound');
            end
            imageName = obj.imageNames{index};
            imageData = obj.getImageDataAtImageName(imageName);
        end
  
       function imageData=getImageDataAtImageName(obj, imageName)
            % Load files
            imagePath = fullfile(obj.datasetPath, imageName, strcat(imageName, '.jpg'));
            load ( fullfile( obj.datasetPath,  imageName, strcat(imageName, 'GroundTruthVP_CamParams') ) );
            load ( fullfile( obj.datasetPath,  imageName, strcat(imageName, 'GroundTruthVP_Orthogonal_CamParams') ) );
            load ( fullfile( obj.datasetPath, imageName, strcat(imageName, 'LinesAndVP') ) );
            imageData.name = imageName;
            imageData.lines = [ lines(1:2:size(lines,1), :) lines(2:2:size(lines,1), :)];
            % Matlab coordinates start at 1 and not 0, we substract 1
            imageData.lines = imageData.lines - ones(size(imageData.lines));
            imageData.vp_association = vp_association;
            imageData.img = imread(imagePath);
            % Compute line equations ax + by + cz = 0 form the lines endpoints
            n = size(imageData.lines, 1);
            linesEq = zeros(n, 3);
            for i=1:n
                linesEq(i,:) = cross([imageData.lines(i,1:2) 1], [imageData.lines(i,3:4) 1]);
            end
            imageData.linesEq = linesEq;
            % Old way to compute the length2
%             length2 = imageData.lines(:,1:2) - imageData.lines(:,3:4);
%             length2 = length2 .* length2;
%             length2 = sum(length2, 2);
%             imageData.lineLength2 = length2;
            % More efficient: 
            imageData.lineLength2 = dot(linesEq(:,1:2)', linesEq(:,1:2)')';
            % Intrinsics
            width = size(imageData.img, 2);        
            height = size(imageData.img, 1);
            imsize = size(imageData.img);
            if (width > height)
                pp = obj.pp;
                ppAcc = [20.52 10.035];
            else
                pp = obj.pp(end:-1:1);
                ppAcc = [10.035 20.52];
            end;
            K = [obj.focal 0 pp(1) ; 0 obj.focal pp(2) ; 0 0 1];
            imageData.K = K;
            imageData.Kinv = inv(K);
            imageData.Kacc = [10.035 0 ppAcc(1) ; 0 10.035 ppAcc(2) ; 0 0 1];
            % VP
            % Swap y axis
            vp = [1, 0, 0 ; 0, -1, 0 ; 0, 0, 1] * vp;
            vp_img = K * vp;
            vp_img = vp_img ./ ([1;1;1] * vp_img(3,:));
            % Same for vp_orthogonal
            vp_orthogonal = [1, 0, 0 ; 0, -1, 0; 0, 0, 1] * vp_orthogonal;
            vp_ortho_img = K * vp_orthogonal;
            vp_ortho_img = vp_ortho_img ./ ([1;1;1] * vp_ortho_img(3,:));
            imageData.vp = vp;
            imageData.vp_img = vp_img;
            imageData.vp_ortho = vp_orthogonal;
            imageData.vp_ortho_img = vp_ortho_img;

       end
    end
end
