function polygons = doubleWedgesIntersection(segments, accuracy)
% Compute the intersection of the double wedges of the segments
% INPUT
%   segments nx4 double (mandatory) represents the n segments. segments(i,:) = [endpt1_x endpt1_y endpt2_x endpt2_y]
%   accuracy 1x1 double (optional)  representes the tolerance on the endpoints of the segment
% OUTPUT
%   polygons 1xn cell array of polygons. With n \in [0, 2]. n=0 when the intersection is empty. n=2 when the polygon is
%            cut by the infinite line [0 0 1]
%            Each polygon is a mx3 double matrix of m points represented in homogeneous coordinates.

    if nargin <= 1
        accuracy = 2;
    end
    
    polygons = {};
    

    if size(segments,1)==0
        return
    end
        
    % We consider the double wedge in the image space and split it in two triangles :
    % [v_inf1 ; midpoint ; v_inf2] and [-v_inf1 ; midpoint ; -v_inf2]
    % Each triangle has two infinite point vertices and one finite vertice.
    [doubleWedge, sepLine] = segmentToDoubleWedge(segments(1,:), accuracy);
    v_inf1 = computeLineInfinitePoint(doubleWedge(1,:)); 
    v_inf2 = computeLineInfinitePoint(doubleWedge(2,:));
    if dot(v_inf1, sepLine) > 0
        v_inf1 = -v_inf1;
    end
    if dot(v_inf2, sepLine) > 0
        v_inf2 = -v_inf2;
    end
    midpoint = [(segments(1,1:2) + segments(1,3:4))/2 1];
    polygons{1} = [v_inf1 ; midpoint ; v_inf2];
    polygons{2} = [-v_inf1 ; midpoint ; -v_inf2];

    % Before entering the loop:  polygons = the two triangles of the double wedges
    % After the first iteration: polygons = the two triangles intersected with the double wedge defined by segments(2,:)
    % After the third iteration: polygons = the two polygons  intersected with the double wedge defined by segments(3,:)
    % ...
    % Invariant: at the ith iteration, the polygons have at most 2*i+3 edges and are convex
    
    % For each segment
    for iSegment=2:size(segments,1)
        segment = segments(iSegment, :);
        [doubleWedge, ~] = segmentToDoubleWedge(segment, accuracy);
        newPolys = {};
        iPoly=1;
        % For each polygon of polygons
        while iPoly <= size(polygons, 2)
            % A double wedge is the union of two intersections of two halplanes.
            % In our case, the double wedge is defined by h1 \inter -h2 \union -h1 \inter h2 (union of 2 triangles)
            % where h1 and h2 are two half planes eg {[x y z]| [a b c].'*[x y z] <= 0}
            % and -h1 and -h2 their complementary eg {[x y z]| [a b c].'*[x y z] > 0}
            % 
            % For each side of the half-plane (we multiply the half plane equation by mSign)
            for mSign = [-1 1]
                % At the end of the body of this loop, newPoly will contain the intersection of polygons{iPoly} with one
                % of the triangles of the double wedge associated with the iSegment(th) segment
                newPoly = polygons{iPoly};
                for iWedge=1:2
                    % Here we compute the intersection between our polygon (newPoly) and the half plane defined by
                    % h = [a b c].' = mSign * doubleWedge(iWedge,:) associated to the iSegment(th) segment.
                    % (x,y,z)' \in h <=> ax + by +cz <= 0
                    if size(newPoly,1) == 0
                        continue;
                    end
                    % At the end of the body of this loop, newPoly2 is either 
                    % the intersection of the 1st half plane * msign with polygons{iPoly}
                    % the intersection of the 1st half plane * msign with polygons{iPoly} intersected with the 2nd half plane
                    newPoly2 = [];
                    % Boolean matrices indicating the vertices in the half plane
                    % in_halfplane(i) = true indicates the ith vertice of newPoly is in the half plane.
                    in_halfplane = dot(newPoly', mSign * repmat(doubleWedge(iWedge,:)',1,size(newPoly,1))) <= 0;

                    % For each vertice of newPoly, we may add it to the new intersection, remove it from the
                    % intersection, or add a new vertice to the intersection
                    n = size(in_halfplane,2);
                    for k=1:n
                        % in1 = true if p1 = newPoly(k,:) is in the half-plane
                        % in2   same for p2 (the successor vertice)
                        in1 = in_halfplane(k);
                        in2 = in_halfplane(mod(k,n)+1);
                        % p1, p2 are the k(th) and k+1(th) vertices in homogeneous coordinates of newPoly
                        p1 = newPoly(k,:);
                        p2 = newPoly(mod(k,n)+1,:);
                        if in1
                            % p1 is in the half-plane, we add it in the intersection
                            newPoly2 = [newPoly2 ; p1];
                        end
                        if in1 ~= in2
                            % The edge [p1 p2] crosses the half-plane. Compute the intersection.
                            inters = cross(mSign * doubleWedge(iWedge,:), cross(p1, p2)); % Only if dot(mSign*sepLine, p1) * dot(mSign*sepLine, p2) == -1
                            if inters(3) < 0
                                % We need to be in the space z>=0 to determine if we are on the left or on the right of a
                                % line. In addition the space z>=0 is the part of the gaussian sphere we can map to the
                                % image space.
                                inters = -inters;
                            end

                            % Handle the case of intersections which are infinite points
                            if abs(inters(3)) < eps
                                if abs(p1(3)) < eps && abs(p2(3)) < eps
                                    % p1(3) == 0 and p2(3) == 0, so (p1 p2) is the infinite line [0 0 1]'
                                    % See the infinite line as the circle z=0
                                    % p1, p2 and inter are on this circle
                                    % Let O be the origin.
                                    % inters is the intersection of the line(p1 p2) with the half plane but we want the
                                    % intersection of the line segment [p1 p2] with the half plane which is either
                                    % inters or -inters
                                    % inters is supposed to be in the sector (p1 O p2), we take the solution which
                                    % statisfies this constraint.
                                    
                                    orientation = dot([-p1(2) p1(1)], p2(1:2));
                                    inSector = orientation*dot([-p1(2) p1(1)], inters(1:2)) >= 0 &&  orientation*dot([-inters(2) inters(1)], p2(1:2)) >= 0;
                                    % If not in the sector, 
                                    if ~inSector
                                        inters = - inters;
                                    end
                                    % Verification
                                    inSector = orientation*dot([-p1(2) p1(1)], inters(1:2)) >= 0 &&  orientation*dot([-inters(2) inters(1)], p2(1:2)) >= 0;
                                    if ~inSector
                                        assert(inSector, 'inters is supposed to be in the sector');
                                    end
                                else
                                    % Case triggered when p1 xor p2 is an infinite point
                                    % Let pp be the extremity of [p1 p2] which is an infinite point.
                                    % It means [p1 p2] is parallel to the half plane boundary
                                    % So inters == pp 
                                    pp = p1;
                                    if abs(p2(3)) < eps
                                        pp = p2;
                                    end
                                    
                                    % Verification
                                    assert(abs(pp(3)) <= eps, 'The case where inters is an infinite point and p1 and p2 are finite points cannot occur since p1 and p2 are in the image space.');
                                    % Explanation, see the image space bounded by the circle of l_infinity [0 0 1]'
                                    % p1 and p2 are in the image space. So if the intersection of [p1 p2] with a
                                    % line is in l_infinity it implies either p1 is finite and p2 is infinite,
                                    % either p1 is infinite and p2 is finite either both p1 and p2 and infinite.    

                                    % Verification inters == pp
                                    assert(det([pp(1:2) ; inters(1:2)]) <= eps, 'We should have inters == pp');
                                    
                                    inters = pp; % May introduce a duplicate vertice if pp is inserted too
                                end
                            end
                            newPoly2 = [newPoly2 ; inters];
                            % In projective geometry, if you never divide (normalize) you can end up with huge numbers
                            newPoly2 = normalize_points(newPoly2);
                            newPoly2 = removeConsecutiveIdenticalPoints(newPoly2);
                        end
                    end
                    newPoly = newPoly2;
                end
                % We normalize the solution
                newPoly = normalize_points(newPoly);
                newPoly = removeConsecutiveIdenticalPoints(newPoly);
                if size(newPoly, 1) > 2
                    newPolys = [newPolys, newPoly];
                end
            end
            iPoly = iPoly + 1;
        end
        polygons = newPolys;
    end
    % For reasonable accuracy and long enough line segments we have size(polygons,2) <= 2
    % However if the double wedges define a "bigger" area than their complementary, we may have more than 2 polygons in
    % the solution.
end

function newPoints = removeConsecutiveIdenticalPoints(points)
    % Remove consecutive identical 2d points represented in homogeneous cordinates.
    % To do so we need to normalize the points in a first step.
    % INPUT
    %   points    nx3   the n points which may contain consecutive duplicate
    % OUTPUT
    %   newPoints mx3   the m points without consecutive duplicate with m<=n
    if size(points, 1) == 0
        newPoints = [];
        return;
    end
    newPoints = points(1,:);
    n = size(points,1);
    for i=2:n
        if norm(points(i,:) - points(mod(i,n)+1,:)) > eps
            newPoints = [newPoints ; points(i,:)];
        end
    end
end


function points = normalize_points(points)
    % Normalize 2d homogeneous points. For each point.
    % if the point is finite, returns the representation with point(3) = 1
    % if the point is infinite, returns the representation with point(2) = 0 if possible ...
    % INPUT
    %   points nx3 double  n points in homogeneous coordinates
    % OUTPUT
    %   points xx3 double  the normalized points
    if size(points, 1) == 0
        return;
    end
    % Non infinite points (3rd coordinate non zero)
    nonInfPts = abs(points(:,3))>eps;
    points(nonInfPts,:) = points(nonInfPts,:) ./ (points(nonInfPts,3) * ones(1,3));
    % Infinite points with 2nd coordinate non zero (the others)
    infPoints = ~nonInfPts;
    infPts_n2 = (abs(points(:,2)) > eps) & infPoints;
    points(infPts_n2,:) = points(infPts_n2,:) ./ (sign(points(infPts_n2,2)) .* points(infPts_n2,2) * ones(1,3));
    % Infinite points with 2nd coordinate zero and 1st coordinate non zero
    infPts_n1 = (abs(points(:,1)) > eps) & ~infPts_n2 & infPoints;
    points(infPts_n1,1) = sign(points(infPts_n1,1));
    % Infinite points with 1st and 2nd coordinates zero
    infPts_1_2 = ~infPts_n2 & ~infPts_n1 & infPoints;
    points(infPts_1_2,3) = sign(points(infPts_1_2,3));
end