function mergedPolygon = mergePolygons(polygons)
    % Merge two polygons splited by the line at infinity into one polygon.
    % We assume we must obtain 1 or 2 polygons (not more)
    % If we have 2 polygons, all these 2 polygons have an edge which is contained in the line at infinity [0 0 1] (in the plane z=0).
    % We assume this edge is their common edges
    % INPUT
    %   polygons: an array of 2 polygons at most with
    %               polygons{1} is a n x 3 matrix
    %               polygons{2} is a m x 3 matrix
    % OUTPUT
    %   mergedPolygon: is a (n+m-2) x 3 matrix representing the merged polygons
    
    assert(size(polygons,2) <= 2, 'The polygon should be splitted in 2 subpolygons at most');
    
    if size(polygons,2) == 0
        mergedPolygon = [];
        return
    end
    
    if size(polygons,2) == 1
        mergedPolygon = polygons{1};
        return
    end 
    
    % We assume:
    %   v1 and v2 are our infinite points
    %   polygons{1} = [v1, v2, ...] after doing a circular shift putting both infinite on 1st and 2nd place
    %   polygons{2} = [..., -v1, -v2, ...] or = [..., -v2, -v1, ...]
    
    % Check each polygon has only 2 infinite points
    nVerts1 = size(polygons{1},1);
    nVerts2 = size(polygons{2},1);
    infPoints1 = abs(polygons{1}(:,3)) < eps;
    assert(size(find(infPoints1), 1) == 2, 'There should be exactly two infinite points in the first subpolygon.');
    infPointsIdx = find(infPoints1);
    % infPointsIdx(1) is the index of v1
    % We do a circular shift to have v1 at the beginning
    polygons{1} = polygons{1}(mod([1:nVerts1]+infPointsIdx(1)-2,nVerts1)+1,:);
    
    if abs(polygons{1}(end,3)) < eps
        % We are in the configuration [v1', ... v2'], we need to make a circular shift to obtain [v2', v1', ...] we
        % rename to [v1, v2, ...]
        % display('cond1')  % For unit test purpose
        polygons{1} = polygons{1}(mod([1:nVerts1]-2,nVerts1)+1,:);
    end
    assert(abs(polygons{1}(1,3)) < eps && abs(polygons{1}(2,3)) < eps);
    
    % Project the polygon in the space z<0 so that we can merge this polygon with the other
    polygons{2} = -polygons{2}(end:-1:1,:); 
    % Search in polygons{2} the index of the row which is equal to v1 and store is in firstVInfIdx
    % v1 = polygons{1}(1,:)
    firstVInfIdx = find(ismember(abs(polygons{2} - ones(nVerts2,1)*polygons{1}(1,:)) < eps, [1 1 1], 'rows'));
    assert(size(firstVInfIdx,1)==1, 'Cannot find an infinite point of poly1 in poly2');
    % Make a circular shift to have v1 at the beginning of p2]
    polygons{2} = polygons{2}(mod([1:nVerts2]+firstVInfIdx-2,nVerts2)+1,:);
    % Now we have either 
    % polygons{2} = [v1, v2, ...] either
    % polygons{2} = [v1, ..., v2]
    infPoints2 = abs(polygons{2}(:,3)) < eps;
    assert(size(find(infPoints2), 1) == 2, 'There should be exactly two infinite points in the second subpolygon.');
    if infPoints2(2)
        % display('cond2') % For unit test purpose
        % We should reverse the polygon, rare but can occur
        polygons{2} = polygons{2}(end:-1:1,:);
        polygons{2} = polygons{2}(mod([1:nVerts2]+nVerts2-2,nVerts2)+1,:);
    end
    % Now we are in the configuration: polygons{2} = [v1, ..., v2] ; polygons{1} = [v1, v2 ...]
    assert(isequal(polygons{1}(1,:), polygons{2}(1,:)));
    assert(isequal(polygons{1}(2,:), polygons{2}(end,:)));
    mergedPolygon = [polygons{2} ; polygons{1}(3:end,:)];
end