function [doubleWedge, sepLine] = segmentToDoubleWedge(segment, accuracy)
% Compute the double wedge associated to a segment wrt to a given accuracy.
% p is in doubleWedge if (dot([doubleWedge ; sepLine]', repmat([1 0 1],3,1)') <= 0) == [1 1 1] or [-1 -1 -1]
% INPUT
%   segment  1x4 double (mandatory) [endpt1_x endpt1_y endpt2_x endpt2_y]
%   accuracy 1x1 double (optional)  representes the tolerance on the endpoints of the segment
% OUTPUT
%   doubleWedge 2x3 double the two half-planes defining the boundaries of the double wedge.
%   sepLine     1x3 the line orthogonal to the segment and passing through its midpoint

    assert(size(segment,1)==1);
    if nargin < 2
        accuracy = 2;
    end
    p1 = segment(1:2);
    p2 = segment(3:4);
    p1h = [p1 1];
    p2h = [p2 1];
    line = cross(p1h, p2h);
    normal = line(1:2);
    normal = normal / norm(normal);
    % Apply the accuracy to the endpoints
    p11 = p1 -  accuracy * normal;
    p12 = p1 +  accuracy * normal;
    p21 = p2 -  accuracy * normal;
    p22 = p2 +  accuracy * normal;
    % Compute the two boundary lines of the double wedge
    sl1h = cross([p11 1], [p22 1]);
    sl2h = cross([p21 1], [p12 1]);
    doubleWedge = [sl1h ; sl2h];
    % Compute sepLine
    orthoVector = [line(1) line(2) 0]; % is perpendicular to the segment
    sepLine = cross((p1h + p2h)/2, orthoVector);
end
